#define	yymaxdepth jags_maxdepth
#define	yyparse	jags_parse
#define	yylex	jags_lex
#define	yyerror	jags_error
#define	yylval	jags_lval
#define	yychar	jags_char
#define	yydebug	jags_debug
#define	yypact	jags_pact
#define	yyr1	jags_r1
#define	yyr2	jags_r2
#define	yydef	jags_def
#define	yychk	jags_chk
#define	yypgo	jags_pgo
#define	yyact	jags_act
#define	yyexca	jags_exca
#define yyerrflag jags_errflag
#define yynerrs	jags_nerrs
#define	yyps	jags_ps
#define	yypv	jags_pv
#define	yys	jags_s
#define	yy_yys	jags_yys
#define	yystate	jags_state
#define	yytmp	jags_tmp
#define	yyv	jags_v
#define	yy_yyv	jags_yyv
#define	yyval	jags_val
#define	yylloc	jags_lloc
#define yyreds	jags_reds
#define yytoks	jags_toks
#define yylhs	jags_yylhs
#define yylen	jags_yylen
#define yydefred jags_yydefred
#define yydgoto	jags_yydgoto
#define yysindex jags_yysindex
#define yyrindex jags_yyrindex
#define yygindex jags_yygindex
#define yytable	 jags_yytable
#define yycheck	 jags_yycheck
#define yyname   jags_yyname
#define yyrule   jags_yyrule
