#include <config.h>
#include <graph/Graph.h>
#include <graph/GraphMarks.h>
#include <graph/Node.h>
#include <graph/StochasticNode.h>
#include <graph/DeterministicNode.h>

#include <stdexcept>
#include <vector>
#include <set>
#include <algorithm>

using std::vector;
using std::set;
using std::invalid_argument;
using std::logic_error;
using std::reverse;

namespace jags {

    Graph::Graph() {}

    bool Graph::contains(Node const *node) const
    {
	return find(const_cast<Node*>(node)) != end();
    }
 
}
